@@ -3,12 +3,36 @@
 setAs("dsyMatrix", "dgeMatrix",
       function(from) .Call("dsyMatrix_as_dgeMatrix", from) )
 
+## I can't get this to work - at least inside Namespace -- FIXME
+## setIs("dgeMatrix", "dsyMatrix",
+##       ## R BUG:  test() doesn't see Matrix-internal functions
+##       test = function(from) Matrix:::isSymmetric(from),
+##       replace = function(obj, value) ## copy all slots
+##       for(n in slotNames(obj)) slot(obj, n) <- slot(value, n)
+##       )
+
 setAs("dsyMatrix", "matrix",
       function(from) .Call("dsyMatrix_as_matrix", from) )
 
 setAs("dsyMatrix", "dspMatrix",
       function(from) .Call("dsyMatrix_as_dspMatrix", from) )
 
+setAs("dsyMatrix", "dsTMatrix",
+      function(from) {
+          ## This is not very efficient (FIXME)
+          ij <- which(as(from,"matrix") != 0, arr.ind = TRUE)
+          new("dsTMatrix", i = ij[,1], j = ij[,2],
+              Dim = from@Dim, Dimnames = from@Dimnames)
+      })
+setAs("dsyMatrix", "dsCMatrix",
+      function(from) callGeneric(as(from, "dsTMatrix")))
+
+
+## Note: Just *because* we have an explicit  dtr -> dge coercion,
+##       show( <ddenseMatrix> ) is not okay, and we need our own:
+setMethod("show", "dsyMatrix", function(object) prMatrix(object))
+
+
 setMethod("rcond", signature(x = "dsyMatrix", type = "character"),
           function(x, type, ...)
           .Call("dsyMatrix_rcond", x, type),
@@ -51,26 +75,20 @@ setMethod("norm", signature(x = "dsyMatrix", type = "missing"),
 ## and vice-versa?
 ## MM: I think yes, since the other part can be filled arbitrarily (wrongly)
 ##WAS setMethod("t", signature(x = "dsyMatrix"), function(x) x)
-setMethod("t", signature(x = "dsyMatrix"),
-	  function(x) {
-	      new("dsyMatrix",
-                  Dim = x@Dim[2:1], Dimnames = x@Dimnames[2:1],
-                  x = as.vector(t(as(x, "matrix"))),
-                  uplo = if (x@uplo == "U") "L" else "U",
-                  rcond = x@rcond)
-          }, valueClass = "dsyMatrix")
+setMethod("t", signature(x = "dsyMatrix"), t_trMatrix,
+          valueClass = "dsyMatrix")
 
+## The following has the severe effect of making
+## "dsyMatrix" a subclass of "dpoMatrix" and since the reverse is
+## by definition of "dpoMatrix", the class-hierarchy gets a *cycle* !
+##
 setIs("dsyMatrix", "dpoMatrix",
       test = function(obj)
-          "try-error" != class(try(.Call("dpoMatrix_chol", obj), TRUE)),
-      replace = function(obj, value) {
-          obj@uplo <- value@uplo
-          obj@rcond <- value@rcond
-          obj@factors <- value@factors
-          obj@x <- value@x
-          obj@Dim <- value@Dim
-          obj@Dimnames <- value@Dimnames
-          obj}
+          "try-error" != class(try(.Call("dpoMatrix_chol", obj), silent=TRUE)),
+      ## MM: The following copying is necessary
+      ## -- but shouldn't it be the default in such a case ??
+      replace = function(obj, value) ## copy all slots
+      for(n in slotNames(obj)) slot(obj, n) <- slot(value, n)
       )
 
 ## Now that we have "chol", we can define  "determinant" methods,
