@@ -9,6 +9,15 @@ setAs("lgTMatrix", "lgCMatrix",
 setAs("lgTMatrix", "matrix",
       function(from) as(as(from, "lgCMatrix"), "matrix"))
 
+setAs("lgTMatrix", "dgTMatrix",
+      function(from)
+      ## more efficient than
+      ## as(as(as(sM, "lgCMatrix"), "dgCMatrix"), "dgTMatrix")
+      new("dgTMatrix", i = from@i, j = from@j,
+          x = rep.int(1, length(from@i)),
+          ## cannot copy factors, but can we use them?
+          Dim = from@Dim, Dimnames= from@Dimnames))
+
 setMethod("t", signature(x = "lgTMatrix"),
 	  function(x) new("lgTMatrix", i = x@j, j = x@i,
 			  Dim = x@Dim[2:1],
