@@ -2,11 +2,24 @@
 
 ### contains = "lsparseMatrix"
 
+setAs("lsCMatrix", "matrix",
+      function(from) as(as(from, "lgCMatrix"), "matrix"))
+
+setAs("lsCMatrix", "lgCMatrix",
+      function(from) .Call("sCMatrix_to_gCMatrix", from, PACKAGE = "Matrix"))
+
+setAs("lsCMatrix", "lsTMatrix",
+      function(from) .Call("Csparse_to_Tsparse", from, PACKAGE = "Matrix"))
+
 setAs("lsCMatrix", "dsCMatrix",
       function(from) new("dsCMatrix", i = from@i, p = from@p,
                          x = rep(1, length(from@i)), uplo = from@uplo,
                          Dim = from@Dim, Dimnames = from@Dimnames))
 
+setAs("lsCMatrix", "dgTMatrix",
+      function(from) callGeneric(as(x, "dsCMatrix")))
+
+## FIXME: should be superfluous now:
 setMethod("image", "lsCMatrix",
           function(x, ...) {
               x <- as(as(x, "dsCMatrix"), "dgTMatrix")
