@@ -1,32 +1,17 @@
 ### Define Methods that can be inherited for all subclasses
 
-## An idea: Coercion between *VIRTUAL* classes
-## -- making sure that result is *actual*!
-
+### Idea: Coercion between *VIRTUAL* classes -- as() chooses "closest" classes
+### ----  should also work e.g. for  dense-triangular --> sparse-triangular !
 ## setAs("denseMatrix", "sparseMatrix",
-##       function(from) {
-##       })
+##        function(from) {
+##            as(as(from, "dgeMatrix")
+##        })
 
 ## setAs("dMatrix", "lMatrix",
 ##       function(from) {
 ##       })
 
 
-## For multiplication operations, sparseMatrix overrides other method
-## selections.  Coerce a ddensematrix argument to a dgeMatrix.
-
-setMethod("%*%", signature(x = "sparseMatrix", y = "ddenseMatrix"),
-          function(x, y) callGeneric(x, as(y, "dgeMatrix")))
-
-setMethod("%*%", signature(x = "ddenseMatrix", y = "sparseMatrix"),
-          function(x, y) callGeneric(as(x, "dgeMatrix"), y))
-
-setMethod("crossprod", signature(x = "sparseMatrix", y = "ddenseMatrix"),
-          function(x, y = NULL) callGeneric(x, as(y, "dgeMatrix")))
-
-setMethod("crossprod", signature(x = "ddenseMatrix", y = "sparseMatrix"),
-          function(x, y = NULL) callGeneric(as(x, "dgeMatrix"), y))
-
 ## "graph" coercions -- this needs the graph package which is currently
 ##  -----               *not* required on purpose
 ## Note: 'undirected' graph <==> 'symmetric' matrix
@@ -75,51 +60,55 @@ setAs("dgTMatrix", "graphNEL",
 
 ### Subsetting -- basic things (drop = "missing") are done in ./Matrix.R
 
-## 1)  dsparse -> dgT
-setMethod("[", signature(x = "dsparseMatrix", i = "index", j = "missing",
-			 drop = "logical"),
-	  function (x, i, j, drop)
-          callGeneric(x = as(x, "dgTMatrix"), i=i, drop=drop))
+### FIXME : we defer to the "*gT" -- conveniently, but not efficient for gC !
 
-setMethod("[", signature(x = "dsparseMatrix", i = "missing", j = "index",
-			 drop = "logical"),
-	  function (x, i, j, drop)
-          callGeneric(x = as(x, "dgTMatrix"), j=j, drop=drop))
+## [dl]sparse -> [dl]gT   -- treat both in one via superclass
+##                        -- more useful when have "z" (complex) and even more
 
-setMethod("[", signature(x = "dsparseMatrix",
+setMethod("[", signature(x = "sparseMatrix", i = "index", j = "missing",
+			 drop = "logical"),
+	  function (x, i, j, drop) {
+              cl <- class(x)
+              viaCl <- if(is(x,"dMatrix")) "dgTMatrix" else "lgTMatrix"
+              x <- callGeneric(x = as(x, viaCl), i=i, drop=drop)
+              ## try_as(x, c(cl, sub("T","C", viaCl)))
+              if(is(x, "Matrix") && extends(cl, "CsparseMatrix"))
+                  as(x, sub("T","C", viaCl)) else x
+          })
+
+setMethod("[", signature(x = "sparseMatrix", i = "missing", j = "index",
+			 drop = "logical"),
+	  function (x, i, j, drop) {
+              cl <- class(x)
+              viaCl <- if(is(x,"dMatrix")) "dgTMatrix" else "lgTMatrix"
+              x <- callGeneric(x = as(x, viaCl), j=j, drop=drop)
+              ## try_as(x, c(cl, sub("T","C", viaCl)))
+              if(is(x, "Matrix") && extends(cl, "CsparseMatrix"))
+                  as(x, sub("T","C", viaCl)) else x
+          })
+
+setMethod("[", signature(x = "sparseMatrix",
 			 i = "index", j = "index", drop = "logical"),
-	  function (x, i, j, drop)
-          callGeneric(x = as(x, "dgTMatrix"), i=i, j=j, drop=drop))
-
-## 2)  lsparse -> lgT
-setMethod("[", signature(x = "lsparseMatrix", i = "index", j = "missing",
-			 drop = "logical"),
-	  function (x, i, j, drop)
-          callGeneric(x = as(x, "lgTMatrix"), i=i, drop=drop))
-
-setMethod("[", signature(x = "lsparseMatrix", i = "missing", j = "index",
-			 drop = "logical"),
-	  function (x, i, j, drop)
-          callGeneric(x = as(x, "lgTMatrix"), j=j, drop=drop))
-
-setMethod("[", signature(x = "lsparseMatrix",
-			 i = "index", j = "index", drop = "logical"),
-	  function (x, i, j, drop)
-          callGeneric(x = as(x, "lgTMatrix"), i=i, j=j, drop=drop))
-
+	  function (x, i, j, drop) {
+              cl <- class(x)
+              viaCl <- if(is(x,"dMatrix")) "dgTMatrix" else "lgTMatrix"
+              x <- callGeneric(x = as(x, viaCl), i=i, j=j, drop=drop)
+              ## try_as(x, c(cl, sub("T","C", viaCl)))
+              if(is(x, "Matrix") && extends(cl, "CsparseMatrix"))
+                  as(x, sub("T","C", viaCl)) else x
+          })
 
 
+setMethod("-", signature(e1 = "sparseMatrix", e2 = "missing"),
+          function(e1) { e1@x <- -e1@x ; e1 })
+## with the following exceptions:
+setMethod("-", signature(e1 = "lsparseMatrix", e2 = "missing"),
+          function(e1) callGeneric(as(e1, "dgCMatrix")))
+setMethod("-", signature(e1 = "pMatrix", e2 = "missing"),
+          function(e1) callGeneric(as(e1, "lgTMatrix")))
 
 ### --- show() method ---
 
-emptyColnames <- function(x)
-{
-    ## Useful for compact printing of (parts) of sparse matrices
-    ## possibly  dimnames(x) "==" NULL :
-    dimnames(x) <- list(dimnames(x)[[1]], rep("", dim(x)[2]))
-    x
-}
-
 prSpMatrix <- function(object, zero.print = ".")
 {
     stopifnot(is(object, "sparseMatrix"))
@@ -163,17 +152,17 @@ setMethod("show", signature(object = "sparseMatrix"),
 
 
 ## not exported:
-setMethod("isSymmetric", signature(object = "sparseMatrix"),
-	  function(object, ...) {
+setMethod("isSymmetric", signature(object = "sparseMatrix", tol = "ANY"),
+	  function(object, tol = 100*.Machine$double.eps) {
 	      ## pretest: is it square?
 	      d <- dim(object)
 	      if(d[1] != d[2]) return(FALSE)
 	      ## else slower test
-	      if (is(object("dMatrix")))
+	      if (is(object, "dMatrix"))
 		  ## use gC; "T" (triplet) is *not* unique!
 		  isTRUE(all.equal(as(object, "dgCMatrix"),
-				   as(t(object), "dgCMatrix"), ...))
-	      else if (is(object("lMatrix")))
+				   as(t(object), "dgCMatrix"), tol = tol))
+	      else if (is(object, "lMatrix"))
 		  ## test for exact equality; FIXME(?): identical() too strict?
 		  identical(as(object, "lgCMatrix"),
 			    as(t(object), "lgCMatrix"))
