@@ -14,10 +14,12 @@ SEXP
     Matrix_RXXSym,
     Matrix_RZXSym,
     Matrix_XtXSym,
+    Matrix_XtySym,
     Matrix_ZZpOSym,
     Matrix_ZZxSym,
     Matrix_ZtXSym,
     Matrix_ZtZSym,
+    Matrix_ZtySym,
     Matrix_bVarSym,
     Matrix_cnamesSym,
     Matrix_devCompSym,
@@ -34,6 +36,8 @@ SEXP
     Matrix_pSym,
     Matrix_permSym,
     Matrix_rcondSym,
+    Matrix_rXySym,
+    Matrix_rZySym,
     Matrix_statusSym,
     Matrix_uploSym,
     Matrix_xSym,
