@@ -1,45 +1,36 @@
-/* Sparse matrices in triplet form */
 #include "dgTMatrix.h"
 
+#ifdef USE_CHOLMOD
+#include "chm_common.h"
+#include "Tsparse.h"
+#endif /* USE_CHOLMOD */
+
 SEXP dgTMatrix_validate(SEXP x)
 {
     SEXP
 	islot = GET_SLOT(x, Matrix_iSym),
-	jslot = GET_SLOT(x, Matrix_jSym),
-	xslot = GET_SLOT(x, Matrix_xSym),
-	dimslot = GET_SLOT(x, Matrix_DimSym);
-    int j,
-	*dims = INTEGER(dimslot),
-	ncol, nrow, nnz = length(islot),
-	*xj = INTEGER(jslot),
-	*xi = INTEGER(islot);
+	xslot = GET_SLOT(x, Matrix_xSym);
 
-    if (length(xslot) != nnz || length(jslot) != nnz)
-	return mkString(_("lengths of slots i, j, and x must match"));
-    if (length(dimslot) != 2)
-	return mkString(_("slot Dim must have length 2"));
-    nrow = dims[0]; ncol = dims[1];
-    for (j = 0; j < nnz; j++) {
-	if (xi[j] < 0 || xi[j] >= nrow)
-	    return mkString(_("all row indices must be between 0 and nrow-1"));
-	if (xj[j] < 0 || xj[j] >= ncol)
-	    return mkString(_("all column indices must be between 0 and ncol-1"));
-    }
+    if (LENGTH(xslot) != LENGTH(islot))
+	return mkString(_("lengths of slots i and x must match"));
     return ScalarLogical(1);
 }
 
 SEXP dgTMatrix_to_dgCMatrix(SEXP x)
 {
+#ifdef USE_CHOLMOD
+    return Tsparse_to_Csparse(x);
+#else
     SEXP dd = GET_SLOT(x, Matrix_DimSym),
 	iP = GET_SLOT(x, Matrix_iSym),
 	ans = PROTECT(NEW_OBJECT(MAKE_CLASS("dgCMatrix")));
     int *dims = INTEGER(dd), nnz = length(iP);
-    int *p, *ti = Calloc(nnz, int), m = dims[0], n = dims[1];
+    int *p = INTEGER(GET_SLOT(ans, Matrix_pSym)),
+        *ti = Calloc(nnz, int), m = dims[0], n = dims[1];
     double *tx = Calloc(nnz, double);
 
     SET_SLOT(ans, Matrix_pSym, allocVector(INTSXP, n + 1));
     SET_SLOT(ans, Matrix_DimSym, duplicate(dd));
-    p = INTEGER(GET_SLOT(ans, Matrix_pSym));
     triplet_to_col(m, n, nnz, INTEGER(iP),
 		   INTEGER(GET_SLOT(x, Matrix_jSym)),
 		   REAL(GET_SLOT(x, Matrix_xSym)),
@@ -53,6 +44,7 @@ SEXP dgTMatrix_to_dgCMatrix(SEXP x)
     Free(ti); Free(tx);
     UNPROTECT(1);
     return ans;
+#endif /* USE_CHOLMOD */
 }
 
 static void
