@@ -10,7 +10,10 @@ dimnames(m5) <- list(Rows= LETTERS[1:5], paste("C", 1:6, sep=""))
 stopifnot(dim(m5) == 5:6,
           class(cm5 <- crossprod(m5)) == "dpoMatrix")
 assert.EQ.mat((c.m5 <- t(m5) %*% m5), as(cm5, "matrix"))
-## but the 'dimnames' are not the same (and are *both*) wrong -- FIXME
+## classes differ; but the 'dimnames' are *both* missing -- FIXME
+tc.m5 <- m5 %*% t(m5)
+(tcm5 <- tcrossprod(m5)) # "dpo*"
+assert.EQ.mat(tc.m5, as(tcm5, "matrix"))# missing dimnames - FIXME
 
 ## right and left "numeric" and "matrix" multiplication:
 (p1 <- m5 %*% c(10, 2:6))
@@ -24,21 +27,21 @@ assert.EQ.mat(p1, cbind(c(20,30,33,38,54)))
 assert.EQ.mat(pd1, as(m5,"matrix") %*% diag(1:6))
 assert.EQ.mat(pd2, diag(10:6) %*% as(m5,"matrix"))
 
-## <FIXME>  -- maybe several things:
-M <- mm[1:500, 1:200] # dgT*
 
-showMethods("%*%", class=class(M)) ## really is empty ``at first''
+M <- mm[1:500, 1:200]
+
+showMethods("%*%", class=class(M))
 
 v1 <- rep(1, ncol(M))
-if(FALSE) ## infinite recursion !
-r <- M %*% Matrix(v1)
-if(FALSE) ## infinite recursion !
-r2 <- Matrix(rep(1,nrow(M))) %*% M
-try(r3 <- M %*% cbind(v1))# dispatches to old ("S3") default method!
-## same result as {but cbind() slightly differs: has dimnames!}:
-try(r3. <- M %*% as(v1, "matrix"))
+str(r <-  M %*% Matrix(v1))
+str(r. <- M %*% cbind(v1))
+stopifnot(identical3(r, r., M %*% as(v1, "matrix")))
+
+v2 <- rep(1,nrow(M))
+r2 <- t(Matrix(v2)) %*% M
+str(r2. <- v2 %*% M)
+stopifnot(identical4(r2, r2., rbind(v2) %*% M, t(as(v2, "matrix")) %*% M))
 
-## </FIXME>
 
 ###--- "logical" Matrices : ---------------------
 
@@ -53,17 +56,16 @@ sM  <- new("lgTMatrix", i = rowi, j=coli, Dim=as.integer(c(N,N)))
 sM # nice
 
 sm <- as(sM, "matrix")
+sM %*% sM
 assert.EQ.mat(sM %*% sM,        sm %*% sm)
-all.equal(as(t(sM) %*% sM, "matrix"),
-          (t(sm) %*% sm) > 0, tol=0)
-
-## is it intended that these are so different?
+assert.EQ.mat(t(sM) %*% sM,
+              (t(sm) %*% sm) > 0, tol=0)
 crossprod(sM)
-t(sM) %*% sM
-
-## These two are *not* TRUE !
-## assert.EQ.mat(crossprod(sM),    crossprod(sm))
-## assert.EQ.mat(tcrossprod(sM),   tcrossprod(sm))
+tcrossprod(sM)
+stopifnot(identical( crossprod(sM), t(sM) %*%   sM),
+          identical(tcrossprod(sM),   sM  %*% t(sM)))
 
+assert.EQ.mat( crossprod(sM),  crossprod(sm) > 0)
+assert.EQ.mat(tcrossprod(sM), as(tcrossprod(sm),"matrix") > 0)
 
 proc.time() # for ``statistical reasons''
